/**
 * WordPress dependencies
 */
const { __ } = wp.i18n;
const { Component } = wp.element;
const { InspectorControls, PanelColorSettings, MediaUpload } = wp.editor;
const {
	PanelBody,
	SelectControl,
	ButtonGroup,
	Button,
	BaseControl,
	IconButton,
	RangeControl
} = wp.components;

/**
 * Internal Dependencies
 */
import {
	CONTENT_STYLES,
	CONTENT_ALIGN,
	BACKGROUND_TYPE,
	HEADER_TAGS,
	BUTTON_SIZES
} from "./constants";
import ImageAvater from "../../../util/image-avatar/ImageAvater";
import FontIconPicker from "@fonticonpicker/react-fonticonpicker";
import iconList from "../../../util/faIcons";
import { StaticIcon } from "../../../util/static-icon";
import GradientColorController from "../../../util/gradient-color-controller";
import DimentionsControl from "../../../util/dimentions-control";
import UnitControl from "../../../util/unit-control";

class Inspector extends Component {
	changeButtonSize = newSize => {
		switch (newSize) {
			case "small":
				this.props.setAttributes({
					buttonPaddingTop: 5,
					buttonPaddingRight: 10,
					buttonPaddingBottom: 5,
					buttonPaddingLeft: 10,
					buttonTextSize: 14,
					buttonSize: newSize
				});
				break;

			case "medium":
				this.props.setAttributes({
					buttonPaddingTop: 10,
					buttonPaddingRight: 20,
					buttonPaddingBottom: 10,
					buttonPaddingLeft: 20,
					buttonTextSize: 16,
					buttonSize: newSize
				});
				break;

			case "large":
				this.props.setAttributes({
					buttonPaddingTop: 10,
					buttonPaddingRight: 30,
					buttonPaddingBottom: 10,
					buttonPaddingLeft: 30,
					buttonTextSize: 24,
					buttonSize: newSize
				});
				break;

			case "extra-large":
				this.props.setAttributes({
					buttonPaddingTop: 10,
					buttonPaddingRight: 40,
					buttonPaddingBottom: 10,
					buttonPaddingLeft: 40,
					buttonTextSize: 32,
					buttonSize: newSize
				});
				break;
		}
	};

	render = () => {
		const { attributes, setAttributes } = this.props;
		const {
			marginTop,
			marginRight,
			marginBottom,
			marginLeft,
			paddingTop,
			paddingRight,
			paddingBottom,
			paddingLeft,
			contentStyle,
			backgroundType,
			backgroundColor,
			backgroundImageURL,
			backgroundImageID,
			icon,
			iconSize,
			iconColor,
			titleTag,
			titleFontSize,
			titleColor,
			descriptionFontSize,
			descriptionColor,
			buttonSize,
			buttonTextSize,
			buttonBackgroundColor,
			buttonTextColor,
			buttonPaddingTop,
			buttonPadddinRight,
			buttonPaddingBottom,
			buttonPaddingLeft,
			buttonBorderRadius,
			shadowColor,
			hOffset,
			vOffset,
			blur,
			spread,
			paddingUnit,
			marginUnit,
			titleSizeUnit,
			descriptionSizeUnit,
			buttonSizeUnit,
			buttonRadiusUnit,
			iconSizeUnit
		} = attributes;

		return (
			<InspectorControls key="controls">
				<PanelBody title={__("Content Settings")}>
					<SelectControl
						label={__("Content Style")}
						value={contentStyle}
						options={CONTENT_STYLES}
						onChange={newStyle => setAttributes({ contentStyle: newStyle })}
					/>

					<BaseControl label={__("Background Type")}>
						<ButtonGroup>
							{BACKGROUND_TYPE.map(item => (
								<Button
									isLarge
									isDefalult={backgroundType !== item.value}
									isPrimary={backgroundType === item.value}
									onClick={() => setAttributes({ backgroundType: item.value })}
								>
									{item.label}
								</Button>
							))}
						</ButtonGroup>
					</BaseControl>

					{backgroundType === "color" && (
						<PanelColorSettings
							title={__("Background Color")}
							initialOpen={false}
							colorSettings={[
								{
									value: backgroundColor,
									onChange: newColor =>
										setAttributes({ backgroundColor: newColor }),
									label: __("Background Color")
								}
							]}
						/>
					)}

					{backgroundType === "gradient" && (
						<PanelBody title={__("Gradient Colors")} initialOpen={false}>
							<GradientColorController
								onChange={backgroundGradient =>
									setAttributes({ backgroundGradient })
								}
							/>
						</PanelBody>
					)}

					{backgroundType === "image" && (
						<MediaUpload
							allowedTypes={["image"]}
							value={backgroundImageID}
							onSelect={media =>
								setAttributes({
									backgroundImageURL: media.url,
									backgroundImageID: media.id
								})
							}
							render={({ open }) =>
								!backgroundImageURL && (
									<IconButton
										className="eb-cia-upload-button"
										label={__("Upload Image")}
										icon="format-image"
										onClick={open}
									/>
								)
							}
						/>
					)}

					{backgroundType === "image" && backgroundImageURL && (
						<ImageAvater
							imageUrl={backgroundImageURL}
							onDeleteImage={() => setAttributes({ backgroundImageURL: null })}
						/>
					)}
				</PanelBody>

				<PanelBody title={__("Margin & Padding")} initialOpen={false}>
					<UnitControl
						selectedUnit={marginUnit}
						unitTypes={[
							{ label: "px", value: "px" },
							{ label: "em", value: "em" },
							{ label: "%", value: "%" }
						]}
						onClick={marginUnit => setAttributes({ marginUnit })}
					/>

					<DimentionsControl
						label={__("Margin")}
						top={marginTop}
						right={marginRight}
						bottom={marginBottom}
						left={marginLeft}
						onChange={({ top, right, bottom, left }) =>
							setAttributes({
								marginTop: top,
								marginRight: right,
								marginBottom: bottom,
								marginLeft: left
							})
						}
					/>

					<UnitControl
						selectedUnit={paddingUnit}
						unitTypes={[
							{ label: "px", value: "px" },
							{ label: "em", value: "em" },
							{ label: "%", value: "%" }
						]}
						onClick={paddingUnit => setAttributes({ paddingUnit })}
					/>

					<DimentionsControl
						label={__("Padding")}
						top={paddingTop || 10}
						right={paddingRight || 30}
						bottom={paddingBottom || 10}
						left={paddingLeft || 30}
						onChange={({ top, right, bottom, left }) =>
							setAttributes({
								paddingTop: top,
								paddingRight: right,
								paddingBottom: bottom,
								paddingLeft: left
							})
						}
					/>
				</PanelBody>

				{contentStyle === "inlineWithIcon" && (
					<PanelBody title={__("Icon Settings")} initialOpen={false}>
						<BaseControl>
							<FontIconPicker
								icons={iconList}
								value={icon}
								onChange={icon => setAttributes({ icon })}
								appendTo="body"
							/>
						</BaseControl>

						<UnitControl
							selectedUnit={iconSizeUnit}
							unitTypes={[
								{ label: "px", value: "px" },
								{ label: "em", value: "em" },
								{ label: "%", value: "%" }
							]}
							onClick={iconSizeUnit => setAttributes({ iconSizeUnit })}
						/>

						<RangeControl
							label={__("Icon Size")}
							value={iconSize}
							onChange={newSize => setAttributes({ iconSize: newSize })}
							min={8}
							max={100}
						/>

						<PanelColorSettings
							title={__("Icon Color")}
							initialOpen={false}
							colorSettings={[
								{
									value: iconColor,
									onChange: newColor => setAttributes({ iconColor: newColor }),
									label: __("Icon Color")
								}
							]}
						/>
					</PanelBody>
				)}

				<PanelBody title={__("Title Settings")} initialOpen={false}>
					<BaseControl label={__("Header Tag")}>
						<ButtonGroup>
							{HEADER_TAGS.map(header => (
								<Button
									isDefault={titleTag !== header.value}
									isPrimary={titleTag === header.value}
									onClick={() => setAttributes({ titleTag: header.value })}
								>
									{header.label}
								</Button>
							))}
						</ButtonGroup>
					</BaseControl>

					<UnitControl
						selectedUnit={titleSizeUnit}
						unitTypes={[
							{ label: "px", value: "px" },
							{ label: "em", value: "em" },
							{ label: "%", value: "%" }
						]}
						onClick={titleSizeUnit => setAttributes({ titleSizeUnit })}
					/>

					<RangeControl
						label={__("Font Size")}
						value={titleFontSize}
						allowReset
						onChange={newSize => setAttributes({ titleFontSize: newSize })}
						min={8}
						max={100}
					/>

					<PanelColorSettings
						title={__("Title Color")}
						initialOpen={false}
						colorSettings={[
							{
								value: titleColor,
								onChange: newColor => setAttributes({ titleColor: newColor }),
								label: __("Title Color")
							}
						]}
					/>
				</PanelBody>

				<PanelBody title={__("Description Settings")} initialOpen={false}>
					<UnitControl
						selectedUnit={descriptionSizeUnit}
						unitTypes={[
							{ label: "px", value: "px" },
							{ label: "em", value: "em" },
							{ label: "%", value: "%" }
						]}
						onClick={descriptionSizeUnit =>
							setAttributes({ descriptionSizeUnit })
						}
					/>

					<RangeControl
						label={__("Font Size")}
						value={descriptionFontSize}
						allowReset
						onChange={newSize =>
							setAttributes({ descriptionFontSize: newSize })
						}
						min={8}
						max={64}
					/>

					<PanelColorSettings
						title={__("Description Color")}
						initialOpen={false}
						colorSettings={[
							{
								value: descriptionColor,
								onChange: newColor =>
									setAttributes({ descriptionColor: newColor }),
								label: __("Description Color")
							}
						]}
					/>
				</PanelBody>

				<PanelBody title={__("Button Settings")} initialOpen={false}>
					<SelectControl
						label={__("Button Size")}
						value={buttonSize}
						options={BUTTON_SIZES}
						onChange={newSize => this.changeButtonSize(newSize)}
					/>

					<UnitControl
						selectedUnit={buttonSizeUnit}
						unitTypes={[
							{ label: "px", value: "px" },
							{ label: "em", value: "em" },
							{ label: "%", value: "%" }
						]}
						onClick={buttonSizeUnit => setAttributes({ buttonSizeUnit })}
					/>

					<RangeControl
						label={__("Font Size")}
						value={buttonTextSize}
						allowReset
						onChange={newSize => setAttributes({ buttonTextSize: newSize })}
						min={8}
						max={100}
					/>

					<UnitControl
						selectedUnit={buttonRadiusUnit}
						unitTypes={[
							{ label: "px", value: "px" },
							{ label: "em", value: "em" },
							{ label: "%", value: "%" }
						]}
						onClick={buttonRadiusUnit => setAttributes({ buttonRadiusUnit })}
					/>

					<RangeControl
						label={__("Border Radius")}
						value={buttonBorderRadius}
						allowReset
						onChange={newSize => setAttributes({ buttonBorderRadius: newSize })}
						min={0}
						max={100}
					/>

					<PanelColorSettings
						title={__("Button Color")}
						initialOpen={false}
						colorSettings={[
							{
								value: buttonBackgroundColor,
								onChange: newColor =>
									setAttributes({ buttonBackgroundColor: newColor }),
								label: __("Button Color")
							},
							{
								value: buttonTextColor,
								onChange: newColor =>
									setAttributes({ buttonTextColor: newColor }),
								label: __("Text Color")
							}
						]}
					/>
				</PanelBody>
				<PanelBody title={__("Box Shadow")} initialOpen={false}>
					<PanelColorSettings
						title={__("Shadow Color")}
						colorSettings={[
							{
								label: "Shadow Color",
								value: shadowColor,
								onChange: shadowColor => setAttributes({ shadowColor })
							}
						]}
					/>

					<RangeControl
						label={__("Horizontal Offset")}
						value={hOffset}
						allowReset
						onChange={hOffset => setAttributes({ hOffset })}
						min={0}
						max={100}
					/>

					<RangeControl
						label={__("Vertical Offset")}
						value={vOffset}
						allowReset
						onChange={vOffset => setAttributes({ vOffset })}
						min={0}
						max={100}
					/>

					<RangeControl
						label={__("Blur")}
						value={blur}
						allowReset
						onChange={blur => setAttributes({ blur })}
						min={0}
						max={100}
					/>

					<RangeControl
						label={__("Spread")}
						value={spread}
						allowReset
						onChange={spread => setAttributes({ spread })}
						min={0}
						max={100}
					/>
				</PanelBody>
			</InspectorControls>
		);
	};
}

export default Inspector;
