/**
 * WordPress dependencies
 */
const { __ } = wp.i18n;
const { RichText, MediaUpload } = wp.editor;
const { IconButton } = wp.components;
const { Fragment } = wp.element;

/**
 * Internal dependencies
 */
import IconBox from "./iconbox.js";
import Inspector from "./inspector";

const Edit = ({ attributes, setAttributes, isSelected }) => {
  const {
    backgroundType,
    backgroundImageURL,
    backgroundColor,
    backgroundGradient,
    imageOrIcon,
    selectedIcon,
    clickableLink,
    header,
    content,
    imageUrl,
    imageId,
    flexDirection,
    order,
    showButton,
    buttonText,
    imageHeight,
    imageWidth,
    number,
    headerTag,
    contentFontSize,
    contentColor,
    headerColor,
    headerTextTransform,
    numberColor,
    numberSize,
    iconSize,
    iconColor,
    buttonColor,
    buttonTextColor,
    iconPaddingTop,
    iconPaddingRight,
    iconPaddingBottom,
    iconPaddingLeft,
    headerPaddingTop,
    headerPaddingRight,
    headerPaddingBottom,
    headerPaddingLeft,
    imageMarginTop,
    imageMarginRight,
    imageMarginBottom,
    imageMarginLeft,
    buttonPaddingTop,
    buttonPaddingRight,
    buttonPaddingBottom,
    buttonPaddingLeft,
    buttonAlign,
    marginTop,
    marginRight,
    marginBottom,
    marginLeft,
    paddingTop,
    paddingRight,
    paddingBottom,
    paddingLeft,
    borderWidth,
    borderColor,
    borderStyle,
    borderRadius,
    shadowColor,
    shadowHOffset,
    shadowVOffset,
    shadowBlur,
    shadowSpread,
    contentSizeUnit,
    iconSizeUnit,
    marginUnit,
    paddingUnit,
    radiusUnit,
    headerPaddingUnit,
    iconPaddingUnit,
    borderWidthUnit
  } = attributes;

  const boxWrapperStyle = {
    flexDirection: flexDirection,
    backgroundImage:
      backgroundType === "image" && backgroundImageURL
        ? `url('${backgroundImageURL}')`
        : (backgroundType === "gradient" && backgroundGradient) || "none",
    backgroundColor:
      backgroundType === "fill"
        ? backgroundColor || "#d7d7d7"
        : backgroundType === "image" || backgroundType === "gradient"
        ? "transparent"
        : undefined,
    backgroundSize:
      backgroundType === "image" && backgroundImageURL ? "cover" : "unset",
    margin: `${marginTop || 0}${marginUnit} ${marginRight ||
      0}${marginUnit} ${marginBottom || 0}${marginUnit} ${marginLeft ||
      0}${marginUnit} `,
    padding: `${paddingTop || 0}${paddingUnit} ${paddingRight ||
      0}${paddingUnit} ${paddingBottom || 0}${paddingUnit} ${paddingLeft ||
      0}${paddingUnit} `,
    border: `${borderWidth ||
      0}${borderWidthUnit} ${borderStyle} ${borderColor || "#000000"}`,
    borderRadius: `${borderRadius || 0}${radiusUnit}`,
    boxShadow: `${shadowHOffset || 0}px ${shadowVOffset || 0}px ${shadowBlur ||
      0}px ${shadowSpread || 0}px ${shadowColor || "#000000"}`
  };

  const imageWrapperStyle = {
    display: imageOrIcon === "image" ? "block" : "none",
    backgroundImage:
      imageOrIcon === "image" && imageUrl ? `url(${imageUrl})` : "none",
    height: `${imageHeight}px`,
    width: `${imageWidth}px`,
    backgroundSize: "cover",
    order: order,
    marginTop: imageMarginTop ? `${imageMarginTop}px` : "10px",
    marginRight: imageMarginRight ? `${imageMarginRight}px` : 0,
    marginBottom: imageMarginBottom ? `${imageMarginBottom}px` : 0,
    marginLeft: imageMarginLeft ? `${imageMarginLeft}px` : 0
  };

  const iconStyle = {
    display: imageOrIcon === "icon" && selectedIcon ? "block" : "none",
    order: order,
    color: iconColor,
    fontSize: `${iconSize || 36}${iconSizeUnit}`,
    paddingTop: `${iconPaddingTop || 0}${iconPaddingUnit}`,
    paddingRight: `${iconPaddingRight || 0}${iconPaddingUnit}`,
    paddingBottom: `${iconPaddingBottom || 0}${iconPaddingUnit}`,
    paddingLeft: `${iconPaddingLeft || 0}${iconPaddingUnit}`
  };

  const numberStyle = {
    display: imageOrIcon === "number" ? "block" : "none",
    order: order,
    color: numberColor,
    fontSize: numberSize
  };

  const headerStyle = {
    textTransform: headerTextTransform,
    color: headerColor,
    paddingTop: `${headerPaddingTop || 0}${headerPaddingUnit}`,
    paddingRight: `${headerPaddingRight || 0}${headerPaddingUnit}`,
    paddingBottom: `${headerPaddingBottom || 0}${headerPaddingUnit}`,
    paddingLeft: `${headerPaddingLeft || 0}${headerPaddingUnit}`
  };

  const contentStyle = {
    fontSize: `${contentFontSize || 18}${contentSizeUnit}`,
    color: contentColor
  };

  const buttonWrapperStyles = {
    display: showButton ? "flex" : "none",
    alignSelf: buttonAlign,
    padding: "10px"
  };

  const linkStyles = {
    textDecoration: "none",
    background: buttonColor || "#0084bc",
    color: buttonTextColor || "#ffffff",
    paddingTop: `${buttonPaddingTop}px`,
    paddingRight: `${buttonPaddingRight}px`,
    paddingBottom: `${buttonPaddingBottom}px`,
    paddingLeft: `${buttonPaddingLeft}px`
  };

  return [
    isSelected && (
      <Inspector attributes={attributes} setAttributes={setAttributes} />
    ),

    // Edit view
    <Fragment>
      <div className="infobox-container" style={boxWrapperStyle}>
        <IconBox selectedIcon={selectedIcon} iconStyle={iconStyle} />

        <div style={imageWrapperStyle}>
          <MediaUpload
            onSelect={media =>
              setAttributes({ imageUrl: media.url, imageId: media.id })
            }
            type="image"
            value={imageId}
            render={({ open }) =>
              !imageUrl && (
                <IconButton
                  className="eb-infobox-image components-button"
                  label={__("Upload Image")}
                  icon="format-image"
                  onClick={open}
                />
              )
            }
          />
        </div>

        <div style={numberStyle}>{isNaN(number) ? "0" : number}</div>

        <RichText
          tagName={headerTag}
          style={headerStyle}
          keepPlaceholderOnFocus
          className="eb-infobox-header"
          value={header}
          placeholder={__("Add Title")}
          onChange={newHeader => setAttributes({ header: newHeader })}
          formattingControls={["italic", "strikethrough", "link"]}
        />

        <span style={contentStyle}>
          <RichText
            className="eb-infobox-content"
            keepPlaceholderOnFocus
            value={content}
            placeholder={__("Add Body")}
            onChange={newContent => setAttributes({ content: newContent })}
          />
        </span>

        <div className="infobox-button" style={buttonWrapperStyles}>
          <a
            href={clickableLink}
            className="eb-infobox-link"
            style={linkStyles}
          >
            {buttonText}
          </a>
        </div>
      </div>
    </Fragment>
  ];
};

export default Edit;
