/**
 * WordPress dependencies
 */
const { __ } = wp.i18n;
const { InspectorControls, PanelColorSettings, MediaUpload } = wp.editor;
const {
	PanelBody,
	IconButton,
	BaseControl,
	TextControl,
	ToggleControl,
	SelectControl,
	RangeControl,
	FontSizePicker
} = wp.components;

/**
 * Internal dependencies
 */
import ImageAvater from "../../../util/image-avatar/ImageAvater";
import { BORDER_STYLES, EFFECTS_LIST, FONT_SIZES } from "./constants";
import DimentionsControl from "../../../util/dimentions-control";
import UnitControl from "../../../util/unit-control";

const Inspector = ({ attributes, setAttributes }) => {
	const {
		effectName,
		header,
		content,
		link,
		imageURL,
		imageHeight,
		imageWidth,
		imageAltTag,
		newWindow,
		headerColor,
		contentColor,
		contentFontSize,
		borderType,
		borderTop,
		borderRight,
		borderBottom,
		borderLeft,
		borderStyle,
		borderColor,
		borderRadius,
		shadowColor,
		hOffset,
		vOffset,
		shadowBlur,
		shadowSpread,
		radiusUnit
	} = attributes;

	return (
		<InspectorControls key="controls">
			<PanelBody title={__("Promo Settings")}>
				<TextControl
					label={__("Header")}
					value={header}
					onChange={header => setAttributes({ header })}
				/>

				<TextControl
					label={__("Content")}
					value={content}
					onChange={content => setAttributes({ content })}
				/>
				<TextControl
					label={__("Link")}
					value={link}
					onChange={link => setAttributes({ link })}
				/>

				<SelectControl
					label={__("Promo Effect")}
					value={effectName}
					options={EFFECTS_LIST}
					onChange={newEffect => setAttributes({ effectName: newEffect })}
				/>

				<BaseControl label={__("Background Image")}>
					<MediaUpload
						onSelect={media =>
							setAttributes({
								imageURL: media.url
							})
						}
						type="image"
						value={imageURL}
						render={({ open }) =>
							!imageURL && (
								<IconButton
									className="eb-cia-upload-button"
									label={__("Upload Image")}
									icon="format-image"
									onClick={open}
								/>
							)
						}
					/>

					{imageURL && (
						<ImageAvater
							imageUrl={imageURL}
							onDeleteImage={() => setAttributes({ imageURL: null })}
						/>
					)}
				</BaseControl>

				{imageURL && (
					<RangeControl
						label={__("Image Height")}
						value={imageHeight}
						allowReset
						onChange={newSize => setAttributes({ imageHeight: newSize })}
						min={200}
						max={600}
					/>
				)}

				{imageURL && (
					<RangeControl
						label={__("Image Width")}
						value={imageWidth}
						allowReset
						onChange={newSize => setAttributes({ imageWidth: newSize })}
						min={0}
						max={600}
					/>
				)}

				{imageURL && (
					<TextControl
						label={__("Image alt tag")}
						value={imageAltTag}
						onChange={newValue => setAttributes({ imageAltTag: newValue })}
					/>
				)}

				<ToggleControl
					label={__("Open in new window")}
					checked={newWindow}
					onChange={() => setAttributes({ newWindow: !newWindow })}
				/>
			</PanelBody>

			<PanelColorSettings
				title={__("Colors")}
				initialOpen={false}
				colorSettings={[
					{
						label: __("Header Color"),
						value: headerColor,
						onChange: newColor => setAttributes({ headerColor: newColor })
					},
					{
						label: __("Content Color"),
						value: contentColor,
						onChange: newColor => setAttributes({ contentColor: newColor })
					}
				]}
			/>

			<PanelBody title={__("Content Font Size")} initialOpen={false}>
				<FontSizePicker
					fontSizes={FONT_SIZES}
					withSlider
					value={contentFontSize}
					onChange={newSize => setAttributes({ contentFontSize: newSize })}
				/>
			</PanelBody>

			<PanelBody title={__("Border Settings")} initialOpen={false}>
				<DimentionsControl
					label={__("Border Width")}
					top={borderTop}
					right={borderRight}
					bottom={borderBottom}
					left={borderLeft}
					onChange={({ top, right, bottom, left }) =>
						setAttributes({
							borderTop: top,
							borderRight: right,
							borderBottom: bottom,
							borderLeft: left
						})
					}
				/>

				<SelectControl
					label={__("Border Type")}
					value={borderType}
					onChange={newType => setAttributes({ borderType: newType })}
				/>

				<SelectControl
					label={__("Border Styles")}
					value={borderStyle}
					options={BORDER_STYLES}
					onChange={newStyle => setAttributes({ borderStyle: newStyle })}
				/>

				<PanelColorSettings
					title={__("Border Color")}
					colorSettings={[
						{
							label: __("Border Color"),
							value: borderColor,
							onChange: newColor => setAttributes({ borderColor: newColor })
						}
					]}
				/>

				<UnitControl
					selectedUnit={radiusUnit}
					unitTypes={[
						{ label: "px", value: "px" },
						{ label: "em", value: "em" },
						{ label: "%", value: "%" }
					]}
					onClick={radiusUnit => setAttributes({ radiusUnit })}
				/>

				<RangeControl
					label={__("Border Radius")}
					value={borderRadius}
					allowReset
					onChange={newSize => setAttributes({ borderRadius: newSize })}
					min={0}
					max={100}
				/>
			</PanelBody>

			<PanelBody title={__("Shadow Settings")} initialOpen={false}>
				<PanelColorSettings
					title={__("Shadow Color")}
					colorSettings={[
						{
							label: __("Shadow Color"),
							value: shadowColor,
							onChange: newColor => setAttributes({ shadowColor: newColor })
						}
					]}
				/>

				<RangeControl
					label={__("Horizontal Offset")}
					value={hOffset}
					allowReset
					onChange={newValue => setAttributes({ hOffset: newValue })}
					min={0}
					max={100}
				/>

				<RangeControl
					label={__("Vertical Offset")}
					value={vOffset}
					allowReset
					onChange={newValue => setAttributes({ vOffset: newValue })}
					min={0}
					max={100}
				/>

				<RangeControl
					label={__("Blur")}
					value={shadowBlur}
					allowReset
					onChange={newValue => setAttributes({ shadowBlur: newValue })}
					min={0}
					max={100}
				/>

				<RangeControl
					label={__("Spread")}
					value={shadowSpread}
					allowReset
					onChange={newValue => setAttributes({ shadowSpread: newValue })}
					min={0}
					max={100}
				/>
			</PanelBody>
		</InspectorControls>
	);
};

export default Inspector;
